# RubyLit - This README is a program!

*Note:* This literary program was extended by [Parker Glynn-Adey](https://pgadey.ca).
Dave, please feel free to edit / delete / remix / re-write anything.
The last bit of [Parse Arguments](#label-Parse+Arguments) really needs some work.
The interesting extension aspects of the extension are explained in the sections:

* [Usage](#label-Usage)
* [Batch Generation](#label-Batch+Generation)  

<hr>

The output of this README.md file is a _program_ that turns documents into:

* <a href="html/README.rb.html">README.rb</a> - a Ruby program
* <a href="html/README.html.html">README.html</a> - a (hideously) formatted HTML document

It's a
[literate program](https://en.wikipedia.org/wiki/Literate_programming)
(wikipedia.org)
and the concept comes from Donald Knuth.
(Update: I've written about what it _felt like_ to make this little program:
[literate-program](http://ratfactor.com/cards/literate-programming)
.)

In order to get the process rolling, there's a non-literate `stage1.rb` script
that does the initial "tangling". You'll see what _that_ means in a moment.

To make this literate programming stuff work, all source is indented. That not
only makes it easy to read in source form, but by chosing that method,
I've also made this document a valid Markdown file, so the README will be
properly formatted when you view it on the Web as HTML output via tools such as
[RepoRat](http://ratfactor.com/repos/reporat/).

The program is executed in three steps:

    <<Parse Arguments>>
    <<Tangle>>
    <<Weave>>

"Parse Arguments" is the boring stuff, to make sure that we comply with [Usage](#label-Usage).
"Tangle" extracts a program (or some other text file) and "Weave" creates the documentation.
Those little `<<bracket things>>` are literate programming macros that
include source code from other sections of the document (identified by
Markdown subheadings).

## Usage

    puts "usage: ruby rubylit.rb INPUT-LIT [OUTPUT] [START]"

RubyLit accepts three arguments `rubylit.rb INPUT-LIT [OUTPUT] [START]`.
The `INPUT-LIT` argument is mandatory and specifies the literate program that RubyLit should process.
It is specified as a literate markdown file without the extension, for example `README` in place of `README.md`.

The arguments `[OUTPUT]` and `[START]` are optional.
If `OUTPUT` is present, then RubyLit will tangle the literate program in to the file `OUTPUT`.
If `OUTPUT` is absent, then RubyLit will produce `INPUT-LIT.rb`.

If the argument `START` is present, then RubyLit will tangle the literate program in to the file `OUTPUT` starting from the segment `START`.
This allows for the possibility of [Batch Generation](#label-Batch+Generation) of files.
The default behaviour, when `START` is absent, is to tangle the entire literate program `INPUT-LIT`.

## Tangle

What's neat about literate programming is that the source can be presented
in any order, so you can explain it however you like. The tangling process
puts it back into order so it can actually run.

The _full_ literate programming concept as imagined by Knuth and implemented
in his initial 'WEB' system not only allows you to include bits of code,
but even lets you define parametric macros, so the literate document is
actually a **meta-language** on top of the underlying programming language!

I've just implemented a crude "include" macro for this demonstration, but that
alone gives me a ton of flexibility!

Here's how I've made that work. I've got a `segments` hash that will store
all of the lines of a "segment" in the literate program. I have the program
begin in a segment identified with the `:start` symbol:

    myseg = :start
    segments = {}
    segments[myseg] = []

Then I loop through all of the lines in the source document and handle
just two special cases. (All other lines are treated as the "document"
part of the literate program and are completely ignored here!)

    File.open(fname_input).each do |line|
        <<Handle segments>>
        <<Handle code lines>>
    end

And after gathering the lines into segments, I recursively follow
the includes to write out the final program to a file:

    <<Write the program>>

## Handle segments

When I see a "## " at the beginning of the line, I know it's a Markdown
level 2 heading, which I'm using to indicate new code segments. They
don't _have_ to include code and even if they do, they don't _have_ to
be used.

Here you can see that I'm setting the current segment to the name of
the heading and initializing a new array to store the code lines:

      if(line.start_with?('## '))
        myseg = line[3..].chomp
        segments[myseg] = []
      end

## Handle code lines

When I see a space at the beginning of the line, I know it's indented source
code. This is extremely strict and not flexible and is just one example of the
non-industrial nature of this demonstration program. :-)

      if(line.start_with?(' '))
        segments[myseg].push(line)
      end

## Write the program

Here's the fun part! I've got this recursive method called `put_lines` that
takes an open destination file, the hash of named code segments (each segment
being an array of lines), and a target segment to print.

I'm looking at each line to see if it's a `<<macro thingy>>` (include request).
If it is, I recurse into the requested segment. Otherwise, I just output the
current line to the file:

    def put_lines(file, segments, sname)
      segments[sname].each do |line|

        if(m = /^\s*<<([^>]+)>>\s*$/.match(line))

          put_lines(file, segments, m[1])

        else

          file.puts line

        end

      end
    end

To start the above recursive process, I open the output file and request the appropriate segment.
If the `START` argument was present as `ARGV[2]` at the time of execution, then we start the recursive process from there.
Otherwise, we start tangling from the beginning of the literate program at the `:start` symbol.

    File.open(fname_output, 'w') do |out|
      if ARGV[2]
        puts "Tangling #{fname_input} to output #{fname_output} from \"#{ARGV[2]}\"."
        put_lines(out, segments, ARGV[2])
      else
        puts "Tangling #{fname_input} to output #{fname_output} from :start."
        put_lines(out, segments, :start)
      end
    end

That's it!

## Weave

The "weave" part of the application is the documentation creation portion.

Since my scheme for this literate program is to encode it as pure Markdown,
I could just rely on an external tool to create the HTML (in fact, that's
probably how you're reading this README right now).

But since Ruby comes with a Markdown parser as part of it's Standard Library,
I figured I might as well include it. The parser and generator are part of
the RDoc (Ruby documentation) module:

    require 'rdoc'

The markdown source is the literate program document (yeah, we read it in
the Tangle process and we'll read it again for Weave):

    data = File.read(fname_input)

Then some boilerplate. RDoc is like a mini-Pandoc in that it can take input
and produce output in a bunch of different formats, and we pay for that
flexibility with some complexity:

    formatter = RDoc::Markup::ToHtml.new(RDoc::Options.new, nil)
    html = RDoc::Markdown.parse(data).accept(formatter)

And then I just write that out to a ".html" file, bookended by start and end document tags:

    File.open("#{fname}.html", 'w') do |out|
      puts "Weaving #{fname_input} to output #{fname}.html."
      out.puts("<html><body>")
      out.print(html)
      out.puts("</body></html>")
    end

And that's it!

## Running it!

To turn this README into a program starts with "stage1", which only includes
the "tangle" part of the process (no documentation output):

    $ ruby stage1.rb README

     Found segment 'Usage'
     Found segment 'Tangle'
     Found segment 'Handle segments'
     Found segment 'Handle code lines'
     Found segment 'Write the program'
     Found segment 'Weave'
     Found segment 'Running it!'
     Found segment 'Bootstrapping'
     Found segment 'Batch Generation'
     Found segment 'Parse Arguments'
     Fetching segment 'start'
     Fetching segment 'Parse Arguments'
     Fetching segment 'Usage'
     Fetching segment 'Tangle'
     Fetching segment 'Handle segments'
     Fetching segment 'Handle code lines'
     Fetching segment 'Write the program'
     Fetching segment 'Weave'

(As you can see, I also gave it some output to help me debug segment names.)

That produces `README.rb`, which is now the Ruby program we've described
above, which can be used to process itself again:

    ruby README.rb README

And running that _again_ proves that we're **fully bootstrapped**. We're
running the output of the README against the README:

    ruby README.rb README

(Note that this part of the document you're reading right now has indented
"code" blocks to show the command line and output. Those are not valid Ruby, so
why is that okay? That's okay because they're never explicitly included in the
program!  The Ruby interpreter never sees them.)

## Bootstrapping

This repo includes two simple literate test programs (Markdown files) I used to
get the intial "stage1" program working:

* `hello.md`
* `hello-segments.md`

When stage1 was done, I copied it to use as the basis for the final
document/program you're reading now. Then I followed the "Running it!" process
exactly as shown above and it worked! :-)

## Batch Generation

RubyLit allows for creating multiple files from a single literate markdown file. 
We call this process batch generation. 
It was inspired by the LaTeX package [docstrip](https://ctan.org/pkg/docstrip).
For example, one can extract a `batch-generation.sh` from this `README`. 
The following script first creates the usual `README.rb`, `README.html`, and then produces a file: `batch-generation.sh`.

    #!/bin/bash
   
    ruby README.rb README 
    ruby README.rb README rubylit.rb 

    ruby rubylit.rb README batch-generation.sh "Batch Generation"

Once you can extract arbitary segments as output files, the sky is the limit.
You can have all sorts of weird recursive things going on.
For example, you can extract other literate programs as Markdown files.
One issue with embedding Markdown files in a literate program is that Markdown cares aboue line initial whitespaces. 
The batch generation system is _just_ a bash script, so we can clean up initial whitespaces with a bit of `sed`.
For example, we can do the following.

    ruby rubylit.rb README markdown-example.md "Markdown Example"
    sed --in-place 's/^    //g' markdown-example.md # clean-up initial whitespace
    ruby rubylit.rb markdown-example hello-world.sh
	

## Markdown Example

We leave finding a use for this weird recursive literary programming as an exercise for the reader.

    # An Embedded Literary Program.

    This is itself a literary program!
    *Wow* it's so meta!

        #!/bin/bash
        echo "Hello, world!"

## Parse Arguments

This is the most boring and hacky part of the program.
Ideally, it would be replaced with something using [OptionParser](https://ruby-doc.org/stdlib-2.4.2/libdoc/optparse/rdoc/OptionParser.html).

    if ARGV[0]
      #puts "ARGV[0] is present: adopting value for fname"
      fname = ARGV[0]
      fname_input = "#{fname}.md"
      if ARGV[1]
        #puts "ARGV[1] is present: adopting value for fname_output"
        fname_output = ARGV[1]
        if ARGV[2]
          #puts "ARGV[2] is present: adopting value for initial_segment"
          initial_segment=ARGV[2]
        else
          #puts "No ARGV[2] is present: assuming default value :start"
          initial_segment=:start
        end
      else
        #puts "No ARGV[1] is present: default value"
        fname_output = "#{fname}.rb"
      end
    else
    	<<Usage>>
	exit 1;
    end
